@@ -15,18 +15,23 @@ use windows::Win32::System::Com::CoInitializeEx;
 use windows::Win32::UI::Accessibility::CUIAutomation;
 use windows::Win32::UI::Accessibility::IUIAutomation;
 use windows::Win32::UI::Accessibility::IUIAutomationElement;
+use windows::Win32::UI::Accessibility::IUIAutomationElement3;
 use windows::Win32::UI::Accessibility::IUIAutomationElementArray;
 use windows::Win32::UI::Accessibility::IUIAutomationTreeWalker;
 use windows::Win32::UI::Accessibility::OrientationType;
+use windows::core::Interface;
 
 use crate::conditions::AndCondition;
+use crate::conditions::ClassNameCondition;
 use crate::conditions::Condition;
+use crate::conditions::ControlTypeCondition;
 use crate::conditions::NameCondition;
 use crate::errors::ERR_NOTFOUND;
 use crate::errors::ERR_TIMEOUT;
 use crate::errors::Error;
 use crate::errors::Result;
-use crate::patterns::IUIPattern;
+use crate::patterns::UIPattern;
+use crate::variants::Variant;
 
 #[derive(Clone, Debug)]
 pub struct UIAutomation {
@@ -35,11 +40,10 @@ pub struct UIAutomation {
 
 impl UIAutomation {
     pub fn new() -> Result<UIAutomation> {
-        let automation: IUIAutomation;
-        unsafe {
+        let automation: IUIAutomation = unsafe {
             CoInitializeEx(null_mut(), COINIT_MULTITHREADED)?;
-            automation = CoCreateInstance(&CUIAutomation, None, CLSCTX_ALL)?;
-        }
+            CoCreateInstance(&CUIAutomation, None, CLSCTX_ALL)?
+        };
 
         Ok(UIAutomation {
             automation
@@ -166,7 +170,7 @@ impl UIElement {
         let control_type = unsafe {
             self.element.CurrentControlType()?
         };
-
+        
         Ok(control_type)
     }
 
@@ -343,7 +347,7 @@ impl UIElement {
             self.element.CurrentControllerFor()?
         };
 
-        to_elements(elements)
+        Self::to_elements(elements)
     }
 
     pub fn get_described_by(&self) -> Result<Vec<UIElement>> {
@@ -351,7 +355,7 @@ impl UIElement {
             self.element.CurrentDescribedBy()?
         };
 
-        to_elements(elements)
+        Self::to_elements(elements)
     }
 
     pub fn get_flows_to(&self) -> Result<Vec<UIElement>> {
@@ -359,7 +363,7 @@ impl UIElement {
             self.element.CurrentFlowsTo()?
         };
 
-        to_elements(elements)
+        Self::to_elements(elements)
     }
 
     pub fn get_provider_description(&self) -> Result<String> {
@@ -378,26 +382,42 @@ impl UIElement {
         Ok(())
     }
 
-    pub fn get_pattern<T: IUIPattern>(&self) -> Result<T> {
+    pub fn get_pattern<T: UIPattern>(&self) -> Result<T> {
         let pattern = unsafe {
             self.element.GetCurrentPattern(T::pattern_id())?
         };
 
         T::new(pattern)
     }
-}
 
-fn to_elements(elements: IUIAutomationElementArray) -> Result<Vec<UIElement>> {
-    let mut arr: Vec<UIElement> = Vec::new();
-    unsafe {
-        for i in 0..elements.Length()? {
-            let elem = UIElement::from(elements.GetElement(i)?);
-            arr.push(elem);
-        }
+    pub fn get_property_value(&self, property_id: i32) -> Result<Variant> {
+        let value = unsafe {
+            self.element.GetCurrentPropertyValue(property_id)?
+        };
+
+        Ok(value.into())
     }
 
-    Ok(arr)
+    pub fn show_context_menu(&self) -> Result<()> {
+        let element3: IUIAutomationElement3 = self.element.cast()?;
+        unsafe {
+            element3.ShowContextMenu()?
+        }
+        
+        Ok(())
+    }
 
+    pub(crate) fn to_elements(elements: IUIAutomationElementArray) -> Result<Vec<UIElement>> {
+        let mut arr: Vec<UIElement> = Vec::new();
+        unsafe {
+            for i in 0..elements.Length()? {
+                let elem = UIElement::from(elements.GetElement(i)?);
+                arr.push(elem);
+            }
+        }
+
+        Ok(arr)
+    }
 }
 
 impl From<IUIAutomationElement> for UIElement {
@@ -514,7 +534,7 @@ impl UIMatcher {
     pub fn new(automation: UIAutomation) -> Self {
         UIMatcher {
             automation,
-            depth: 5,
+            depth: 7,
             from: None,
             condition: None,
             timeout: 3000,
@@ -570,11 +590,21 @@ impl UIMatcher {
         self.filter(Box::new(condition))
     }
 
-    pub fn find_first(&self) -> Result<UIElement> {
-        // let (root, walker) = self.prepare()?;
+    pub fn classname<S: Into<String>>(self, classname: S) -> Self {
+        let condition = ClassNameCondition {
+            classname: classname.into()
+        };
+        self.filter(Box::new(condition))        
+    }
 
-        // let mut elements: Vec<UIElement> = Vec::new();
-        // self.search(&walker, &root, &mut elements, 1, true)?;
+    pub fn control_type(self, control_type: i32) -> Self {
+        let condition = ControlTypeCondition {
+            control_type
+        };
+        self.filter(Box::new(condition))
+    }
+
+    pub fn find_first(&self) -> Result<UIElement> {
         let elements = self.find(true)?;
 
         if elements.is_empty() {
